@@ -1,6 +1,8 @@
 from vtkmodules import web
 from vtkmodules.web import render_window_serializer
 from vtkmodules.vtkFiltersGeometry import vtkDataSetSurfaceFilter
+from vtkmodules.vtkCommonMath import vtkMatrix4x4
+from vtkmodules.vtkRenderingCore import vtkColorTransferFunction
 
 # Patch support for BigInt64 (VTK_LONG_LONG)
 # This requires the latest VTK.js
@@ -430,6 +432,160 @@ def genericVolumeMapperSerializer(parent, mapper, mapperId, context, depth):
     return None
 
 
+def axesActorSerializer(parent, actor, actorId, context, depth):
+    actorVisibility = actor.GetVisibility()
+
+    if not actorVisibility:
+        return None
+
+    # C++ extract
+    label_show = actor.GetAxisLabels()
+    # label_position = actor.GetNormalizedLabelPosition()
+
+    # shaft_length = actor.GetNormalizedShaftLength()
+    shaft_type = actor.GetShaftType()  # int [line/cylinder]
+
+    tip_length = actor.GetNormalizedTipLength()
+    # tip_type = actor.GetTipType() # int [cone/sphere]
+
+    cone_resolution = actor.GetConeResolution()
+    cone_radius = actor.GetConeRadius()
+
+    cylinder_resolution = actor.GetCylinderResolution()
+    cylinder_radius = actor.GetCylinderRadius()
+
+    # sphere_radius = actor.GetSphereRadius()
+    # sphere_resolution = actor.GetSphereResolution()
+
+    # XYZ...
+    # actor.GetXAxisCaptionActor2D()
+    # actor.GetXAxisTipProperty()
+    # actor.GetXAxisShaftProperty()
+    # actor.GetXAxisLabelText()
+
+    # Apply transform
+    matrix = vtkMatrix4x4()
+    actor.GetUserTransform().GetTranspose(matrix)
+    user_matrix = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
+    for i in range(4):
+        for j in range(4):
+            idx = i + 4 * j
+            user_matrix[idx] = matrix.GetElement(j, i)
+
+    return {
+        "parent": render_window_serializer.getReferenceId(parent),
+        "id": actorId,
+        "type": "vtkAxesActor",
+        "properties": {
+            # vtkProp
+            "visibility": actorVisibility,
+            "pickable": actor.GetPickable(),
+            "dragable": actor.GetDragable(),
+            "useBounds": actor.GetUseBounds(),
+            # vtkProp3D
+            "origin": actor.GetOrigin(),
+            "position": actor.GetPosition(),
+            "scale": actor.GetScale(),
+            "userMatrix": user_matrix,
+            # vtkAxesActor
+            "labels": {
+                "show": label_show,
+                "x": actor.GetXAxisLabelText(),
+                "y": actor.GetYAxisLabelText(),
+                "z": actor.GetZAxisLabelText(),
+            },
+            "config": {
+                "recenter": 0,
+                "tipResolution": cone_resolution,  # 60,
+                "tipRadius": 0.2 * cone_radius,  # 0.1,
+                "tipLength": tip_length[0],  # 0.2,
+                "shaftResolution": cylinder_resolution,  # 60,
+                "shaftRadius": 0.01 if shaft_type else cylinder_radius,  # 0.03,
+                "invert": 0,
+            },
+            "xAxisColor": list(
+                map(lambda x: int(x * 255), actor.GetXAxisTipProperty().GetColor())
+            ),
+            "yAxisColor": list(
+                map(lambda x: int(x * 255), actor.GetYAxisTipProperty().GetColor())
+            ),
+            "zAxisColor": list(
+                map(lambda x: int(x * 255), actor.GetZAxisTipProperty().GetColor())
+            ),
+        },
+    }
+
+
+def colorTransferFunctionSerializer(parent, instance, objId, context, depth):
+    nodes = []
+    for i in range(instance.GetSize()):
+        # x, r, g, b, midpoint, sharpness
+        node = [0, 0, 0, 0, 0, 0]
+        instance.GetNodeValue(i, node)
+        nodes.append(node)
+
+    discretize = 0
+    numberOfValues = instance.GetSize()
+    if hasattr(instance, "GetDiscretize"):
+        discretize = (
+            instance.GetDiscretize() if hasattr(instance, "GetDiscretize") else 0
+        )
+        numberOfValues = (
+            instance.GetNumberOfValues()
+            if hasattr(instance, "GetNumberOfValues")
+            else 256
+        )
+    elif numberOfValues < 256:
+        discretize = 1
+
+    return {
+        "parent": render_window_serializer.getReferenceId(parent),
+        "id": objId,
+        "type": render_window_serializer.class_name(instance),
+        "properties": {
+            "clamping": 1 if instance.GetClamping() else 0,
+            "colorSpace": instance.GetColorSpace(),
+            "hSVWrap": 1 if instance.GetHSVWrap() else 0,
+            # 'nanColor': instance.GetNanColor(),                  # Breaks client
+            # 'belowRangeColor': instance.GetBelowRangeColor(),    # Breaks client
+            # 'aboveRangeColor': instance.GetAboveRangeColor(),    # Breaks client
+            # 'useAboveRangeColor': 1 if instance.GetUseAboveRangeColor() else 0,
+            # 'useBelowRangeColor': 1 if instance.GetUseBelowRangeColor() else 0,
+            "allowDuplicateScalars": 1 if instance.GetAllowDuplicateScalars() else 0,
+            "alpha": instance.GetAlpha(),
+            "vectorComponent": instance.GetVectorComponent(),
+            "vectorSize": instance.GetVectorSize(),
+            "vectorMode": instance.GetVectorMode(),
+            "indexedLookup": instance.GetIndexedLookup(),
+            "nodes": nodes,
+            "numberOfValues": numberOfValues,
+            "discretize": discretize,
+        },
+    }
+
+
+def lookupTableToColorTransferFunction(lookupTable):
+    dataTable = lookupTable.GetTable()
+    table = render_window_serializer.dataTableToList(dataTable)
+
+    if not table:
+        lookupTable.Build()
+        table = render_window_serializer.dataTableToList(dataTable)
+
+    if table:
+        ctf = vtkColorTransferFunction()
+        ctf.DeepCopy(lookupTable)  # <== needed to capture vector props
+
+        tableRange = lookupTable.GetTableRange()
+        points = render_window_serializer.linspace(*tableRange, num=len(table))
+        for x, rgba in zip(points, table):
+            ctf.AddRGBPoint(x, *[x / 255 for x in rgba[:3]])
+
+        return ctf
+
+    return None
+
+
 def registerAddOnSerializers():
     # Override extractRequiredFields to fix handling of Normals/TCoords
     setattr(render_window_serializer, "extractRequiredFields", extractRequiredFields)
@@ -446,6 +602,16 @@ def registerAddOnSerializers():
         "genericVolumeMapperSerializer",
         genericVolumeMapperSerializer,
     )
+    setattr(
+        render_window_serializer,
+        "colorTransferFunctionSerializer",
+        colorTransferFunctionSerializer,
+    )
+    setattr(
+        render_window_serializer,
+        "lookupTableToColorTransferFunction",
+        lookupTableToColorTransferFunction,
+    )
 
     for name in [
         "vtkMapper",
@@ -465,7 +631,6 @@ def registerAddOnSerializers():
         "vtkOpenGLGPUVolumeRayCastMapper",
         "vtkSmartVolumeMapper",
     ]:
-
         render_window_serializer.registerInstanceSerializer(
             name, genericVolumeMapperSerializer
         )
@@ -479,3 +644,6 @@ def registerAddOnSerializers():
     render_window_serializer.registerInstanceSerializer(
         "vtkImageData", imagedataSerializer
     )
+    render_window_serializer.registerInstanceSerializer(
+        "vtkAxesActor", axesActorSerializer
+    )
